<?php

namespace Litecms\Description\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Description\Http\Requests\DescriptionRequest;
use Litecms\Description\Interfaces\DescriptionRepositoryInterface;
use Litecms\Description\Models\Description;

/**
 * Resource controller class for description.
 */
class DescriptionResourceController extends BaseController
{

    /**
     * Initialize description resource controller.
     *
     * @param type DescriptionRepositoryInterface $description
     *
     * @return null
     */
    public function __construct(DescriptionRepositoryInterface $description)
    {
        parent::__construct();
        $this->repository = $description;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Description\Repositories\Criteria\DescriptionResourceCriteria::class);
    }

    /**
     * Display a list of description.
     *
     * @return Response
     */
    public function index(DescriptionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Description\Repositories\Presenter\DescriptionPresenter::class)
                ->$function();
        }

        $descriptions = $this->repository->paginate();

        return $this->response->title(trans('description::description.names'))
            ->view('description::description.index', true)
            ->data(compact('descriptions'))
            ->output();
    }

    /**
     * Display description.
     *
     * @param Request $request
     * @param Model   $description
     *
     * @return Response
     */
    public function show(DescriptionRequest $request, Description $description)
    {

        if ($description->exists) {
            $view = 'description::description.show';
        } else {
            $view = 'description::description.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('description::description.name'))
            ->data(compact('description'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new description.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DescriptionRequest $request)
    {

        $description = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('description::description.name')) 
            ->view('description::description.create', true) 
            ->data(compact('description'))
            ->output();
    }

    /**
     * Create new description.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DescriptionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $description                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('description::description.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('description/description/' . $description->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/description/description'))
                ->redirect();
        }

    }

    /**
     * Show description for editing.
     *
     * @param Request $request
     * @param Model   $description
     *
     * @return Response
     */
    public function edit(DescriptionRequest $request, Description $description)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('description::description.name'))
            ->view('description::description.edit', true)
            ->data(compact('description'))
            ->output();
    }

    /**
     * Update the description.
     *
     * @param Request $request
     * @param Model   $description
     *
     * @return Response
     */
    public function update(DescriptionRequest $request, Description $description)
    {
        try {
            $attributes = $request->all();

            $description->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('description::description.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('description/description/' . $description->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('description/description/' . $description->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the description.
     *
     * @param Model   $description
     *
     * @return Response
     */
    public function destroy(DescriptionRequest $request, Description $description)
    {
        try {

            $description->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('description::description.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('description/description/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('description/description/' . $description->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple description.
     *
     * @param Model   $description
     *
     * @return Response
     */
    public function delete(DescriptionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('description::description.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('description/description'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/description/description'))
                ->redirect();
        }

    }

    /**
     * Restore deleted descriptions.
     *
     * @param Model   $description
     *
     * @return Response
     */
    public function restore(DescriptionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('description::description.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/description/description'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/description/description/'))
                ->redirect();
        }

    }

}
